@@ -54,7 +54,9 @@ NS_IMETHODIMP GPrintingPromptService::ShowPrintDialog(nsIDOMWindow *parent, nsIW
 	nsresult rv = NS_OK;
 
 	GtkWidget *gtkParent = MozillaFindGtkParent(parent);
+	if (!gtkParent) return NS_ERROR_FAILURE;
 	EphyEmbed *embed = EPHY_EMBED (MozillaFindEmbed (parent));
+	if (!embed) return NS_ERROR_FAILURE;
 
 	dialog = print_dialog_new_with_parent (gtkParent, embed, &info);
 	ephy_dialog_set_modal (dialog, TRUE);
