@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-topic-action.c,v 1.15 2003/05/17 12:43:34 cneumair Exp $
+ *  $Id: ephy-topic-action.c,v 1.25.2.1 2003/12/18 12:48:45 chpe Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -25,10 +25,12 @@
 #include "ephy-node-common.h"
 #include "ephy-topic-action.h"
 #include "ephy-bookmarks.h"
+#include "ephy-favicon-cache.h"
 #include "ephy-shell.h"
 #include "eggtoolitem.h"
 #include "ephy-debug.h"
 #include "ephy-gui.h"
+#include "ephy-string.h"
 
 static void ephy_topic_action_init       (EphyTopicAction *action);
 static void ephy_topic_action_class_init (EphyTopicActionClass *class);
@@ -122,6 +124,7 @@ static void
 menu_deactivate_cb (GtkMenuShell *ms, GtkWidget *button)
 {
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), FALSE);
+	gtk_button_released (GTK_BUTTON (button));
 }
 
 static void
@@ -130,7 +133,7 @@ menu_activate_cb (GtkWidget *item, EggAction *action)
 	EphyNode *node;
 	const char *location;
 
-	node = EPHY_NODE (g_object_get_data (G_OBJECT (item), "node"));
+	node = g_object_get_data (G_OBJECT (item), "node");
 	location = ephy_node_get_property_string
 		(node, EPHY_NODE_BMK_PROP_LOCATION);
 	g_signal_emit (action, ephy_topic_action_signals[GO_LOCATION],
@@ -140,14 +143,30 @@ menu_activate_cb (GtkWidget *item, EggAction *action)
 static void
 ephy_topic_action_sync_label (EggAction *action, GParamSpec *pspec, GtkWidget *proxy)
 {
-	GtkLabel *label;
+	GtkWidget *label = NULL;
 
 	LOG ("Set bookmark action proxy label to %s", action->label)
 
-	label = GTK_LABEL (g_object_get_data (G_OBJECT (proxy), "label"));
+	if (EGG_IS_TOOL_ITEM (proxy))
+	{
+		label = GTK_WIDGET (g_object_get_data (G_OBJECT (proxy), "label"));
+	}
+	else if (GTK_IS_MENU_ITEM (proxy))
+	{
+		label = GTK_BIN (proxy)->child;
+	}
+	else
+	{
+		g_warning ("Unknown widget");
+		return;
+	}
+
 	g_return_if_fail (label != NULL);
 
-	gtk_label_set_label (label, action->label);
+	if (action->label)
+	{
+		gtk_label_set_label (GTK_LABEL (label), action->label);
+	}
 }
 
 static int
@@ -155,40 +174,51 @@ sort_bookmarks (gconstpointer a, gconstpointer b)
 {
 	EphyNode *node_a = (EphyNode *)a;
 	EphyNode *node_b = (EphyNode *)b;
-	char *str_a = NULL;
-	char *str_b = NULL;
+	const char *title1, *title2;
 	int retval;
 
-	g_return_val_if_fail (EPHY_IS_NODE (node_a), 1);
-	g_return_val_if_fail (EPHY_IS_NODE (node_b), -1);
+	title1 = ephy_node_get_property_string (node_a, EPHY_NODE_BMK_PROP_TITLE);
+	title2 = ephy_node_get_property_string (node_b, EPHY_NODE_BMK_PROP_TITLE);
+                                                                                                                      
+	if (title1 == NULL)
+	{
+		retval = -1;
+	}
+	else if (title2 == NULL)
+	{
+		retval = 1;
+	}
+	else
+	{
+		char *str_a, *str_b;
+
+		str_a = g_utf8_casefold (title1, -1);
+		str_b = g_utf8_casefold (title2, -1);
+		retval = g_utf8_collate (str_a, str_b);
+		g_free (str_a);
+		g_free (str_b);
+	}
 
-	str_a = g_utf8_casefold (ephy_node_get_property_string (node_a, EPHY_NODE_BMK_PROP_TITLE),
-				 -1);
-	str_b = g_utf8_casefold (ephy_node_get_property_string (node_b, EPHY_NODE_BMK_PROP_TITLE),
-			         -1);
-	retval = g_utf8_collate (str_a, str_b);
-	g_free (str_a);
-	g_free (str_b);
-	
 	return retval;
 }
 
-static GtkWidget *
-build_bookmarks_menu (EphyTopicAction *action, EphyNode *node)
+#define MAX_LENGTH 32
+
+static void
+append_bookmarks_menu (EphyTopicAction *action, GtkWidget *menu, EphyNode *node, gboolean show_empty)
 {
         EphyFaviconCache *cache;
-	GtkWidget *menu, *item;
+	GtkWidget *item;
 	GPtrArray *children;
 
-	menu = gtk_menu_new ();
-
-        cache = ephy_embed_shell_get_favicon_cache
-               (EPHY_EMBED_SHELL (ephy_shell));
+        cache = EPHY_FAVICON_CACHE
+		(ephy_embed_shell_get_favicon_cache (EPHY_EMBED_SHELL (ephy_shell)));
 
 	children = ephy_node_get_children (node);
 
-	if (children->len < 1)
+	if (children->len < 1 && show_empty)
 	{
+		/* This is the adjective, not the verb */
 		item = gtk_menu_item_new_with_label (_("Empty"));
 		gtk_widget_set_sensitive (item, FALSE);
 		gtk_widget_show (item);
@@ -198,36 +228,38 @@ build_bookmarks_menu (EphyTopicAction *action, EphyNode *node)
 	{
 		GList *node_list = NULL, *l;
 		int i;
-		
+
 		for (i = 0; i < children->len; ++i)
 		{
-			node_list = g_list_append (node_list,
+			node_list = g_list_prepend (node_list,
 						   g_ptr_array_index (children, i));
 		}
-		
-		g_list_sort (node_list, (GCompareFunc)sort_bookmarks);
-	
+
+		node_list = g_list_sort (node_list, (GCompareFunc)sort_bookmarks);
+
 		for (l = g_list_first (node_list); l != NULL; l = g_list_next (l))
 		{
 			EphyNode *kid;
-                	const char *icon_location;
+			const char *icon_location;
 			const char *title;
-	
+			char *title_short;
+
 			kid = (EphyNode*)l->data;
-	
+
 			icon_location = ephy_node_get_property_string
 				(kid, EPHY_NODE_BMK_PROP_ICON);
 			title = ephy_node_get_property_string
 				(kid, EPHY_NODE_BMK_PROP_TITLE);
 			if (title == NULL) continue;
-			LOG ("Create menu for bookmark %s", title)
-	
-			item = gtk_image_menu_item_new_with_label (title);
+			title_short = ephy_string_shorten (title, MAX_LENGTH);
+			LOG ("Create menu for bookmark %s", title_short)
+
+			item = gtk_image_menu_item_new_with_label (title_short);
 			if (icon_location)
 			{
 				GdkPixbuf *icon;
 				GtkWidget *image;
-	
+
 				icon = ephy_favicon_cache_get (cache, icon_location);
 				if (icon != NULL)
 				{
@@ -244,12 +276,22 @@ build_bookmarks_menu (EphyTopicAction *action, EphyNode *node)
 					  G_CALLBACK (menu_activate_cb), action);
 			gtk_widget_show (item);
 			gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+
+			g_free (title_short);
 		}
 
 		g_list_free (node_list);
 	}
-	
-	ephy_node_thaw (node);
+}
+
+static GtkWidget *
+build_bookmarks_menu (EphyTopicAction *action, EphyNode *node)
+{
+	GtkWidget *menu;
+
+	menu = gtk_menu_new ();
+
+	append_bookmarks_menu (action, menu, node, TRUE);
 
 	return menu;
 }
@@ -259,36 +301,31 @@ sort_topics (gconstpointer a, gconstpointer b)
 {
 	EphyNode *node_a = (EphyNode *)a;
 	EphyNode *node_b = (EphyNode *)b;
-	EphyNodePriority priority_a, priority_b;
-	char *str_a = NULL;
-	char *str_b = NULL;
+	const char *title1, *title2;
 	int retval;
 
-	g_return_val_if_fail (EPHY_IS_NODE (node_a), 1);
-	g_return_val_if_fail (EPHY_IS_NODE (node_b), -1);
-
-	priority_a = ephy_node_get_property_int (node_a, EPHY_NODE_KEYWORD_PROP_PRIORITY);	
-	priority_b = ephy_node_get_property_int (node_b, EPHY_NODE_KEYWORD_PROP_PRIORITY);	
-
-	if (priority_a < priority_b)
+	title1 = ephy_node_get_property_string (node_a, EPHY_NODE_KEYWORD_PROP_NAME);
+	title2 = ephy_node_get_property_string (node_b, EPHY_NODE_KEYWORD_PROP_NAME);
+                                                                                                                      
+	if (title1 == NULL)
 	{
 		retval = -1;
 	}
-	else if (priority_a > priority_b)
+	else if (title2 == NULL)
 	{
 		retval = 1;
 	}
 	else
 	{
-		str_a = g_utf8_casefold (ephy_node_get_property_string (node_a, EPHY_NODE_KEYWORD_PROP_NAME),
-					 -1);
-		str_b = g_utf8_casefold (ephy_node_get_property_string (node_b, EPHY_NODE_KEYWORD_PROP_NAME),
-				         -1);
+		char *str_a, *str_b;
+
+		str_a = g_utf8_casefold (title1, -1);
+		str_b = g_utf8_casefold (title2, -1);
 		retval = g_utf8_collate (str_a, str_b);
 		g_free (str_a);
 		g_free (str_b);
 	}
-	
+
 	return retval;
 }
 
@@ -299,9 +336,8 @@ build_topics_menu (EphyTopicAction *action, EphyNode *node)
 	GPtrArray *children;
 	int i;
 	EphyBookmarks *bookmarks;
-	EphyNode *all;
+	EphyNode *all, *uncategorized;
 	EphyNodePriority priority;
-	char *markup_title = NULL;
 	GList *node_list = NULL, *l = NULL;
 
 	bookmarks = ephy_shell_get_bookmarks (ephy_shell);
@@ -310,15 +346,22 @@ build_topics_menu (EphyTopicAction *action, EphyNode *node)
 	menu = gtk_menu_new ();
 
 	children = ephy_node_get_children (node);
-	
+
 	for (i = 0; i < children->len; ++i)
 	{
-		node_list = g_list_append (node_list,
-					   g_ptr_array_index (children, i));
+		EphyNode *kid;
+
+		kid = g_ptr_array_index (children, i);
+		priority = ephy_node_get_property_int
+			(kid, EPHY_NODE_KEYWORD_PROP_PRIORITY);
+		if (priority == EPHY_NODE_NORMAL_PRIORITY)
+		{
+			node_list = g_list_prepend (node_list, kid);
+		}
 	}
-	
-	g_list_sort (node_list, (GCompareFunc)sort_topics);
-	
+
+	node_list = g_list_sort (node_list, (GCompareFunc)sort_topics);
+
 	for (l = g_list_first (node_list); l != NULL; l = g_list_next (l))
 	{
 		EphyNode *kid;
@@ -330,21 +373,9 @@ build_topics_menu (EphyTopicAction *action, EphyNode *node)
 
 		title = ephy_node_get_property_string
 			(kid, EPHY_NODE_KEYWORD_PROP_NAME);
-		priority = ephy_node_get_property_int 
-			(kid, EPHY_NODE_KEYWORD_PROP_PRIORITY);
 		LOG ("Create menu for topic %s", title);
 
-		if (priority == EPHY_NODE_ALL_PRIORITY ||
-		    priority == EPHY_NODE_SPECIAL_PRIORITY)
-		{
-			markup_title = g_strconcat ("<b>", title, "</b>", NULL);
-		}
-		else
-		{
-			markup_title = g_strdup (title);
-		}
-		item = gtk_image_menu_item_new_with_label (markup_title);
-		g_free (markup_title);
+		item = gtk_image_menu_item_new_with_label (title);
 
 		label = gtk_bin_get_child (GTK_BIN (item));
 		gtk_label_set_use_markup  (GTK_LABEL (label), TRUE);
@@ -356,9 +387,11 @@ build_topics_menu (EphyTopicAction *action, EphyNode *node)
 		gtk_widget_show (bmk_menu);
 		gtk_menu_item_set_submenu (GTK_MENU_ITEM (item), bmk_menu);
 	}
-	ephy_node_thaw (node);
 	g_list_free (node_list);
 
+	uncategorized = ephy_bookmarks_get_not_categorized (bookmarks);
+	append_bookmarks_menu (action, menu, uncategorized, FALSE);
+
 	return menu;
 }
 
@@ -366,21 +399,20 @@ static GtkWidget *
 build_menu (EphyTopicAction *action)
 {
 	EphyNode *node;
+	EphyBookmarks *bookmarks;
 
+	bookmarks = ephy_shell_get_bookmarks (ephy_shell);
 
 	if (action->priv->topic_id == BOOKMARKS_NODE_ID)
 	{
-		EphyBookmarks *bookmarks;
-
 		LOG ("Build all bookmarks crap menu")
 
-		bookmarks = ephy_shell_get_bookmarks (ephy_shell);
 		node = ephy_bookmarks_get_keywords (bookmarks);
 		return build_topics_menu (action, node);
 	}
 	else
 	{
-		node = ephy_node_get_from_id (action->priv->topic_id);
+		node = ephy_bookmarks_get_from_id (bookmarks, action->priv->topic_id);
 		return build_bookmarks_menu (action, node);
 	}
 }
@@ -400,7 +432,50 @@ button_toggled_cb (GtkWidget *button,
 				ephy_gui_menu_position_under_widget,
 				button, 1, gtk_get_current_event_time ());
 	}
+}
 
+static void
+button_pressed_cb (GtkWidget *button,
+		   EphyTopicAction *action)
+{
+	 gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);
+}
+
+static GtkWidget *
+create_menu_item (EggAction *action)
+{
+	GtkWidget *menu, *menu_item;
+
+	LOG ("create_menu_item action %p", action)
+
+	menu_item = gtk_menu_item_new_with_label (action->label);
+
+	menu = build_menu (EPHY_TOPIC_ACTION (action));
+	gtk_widget_show (menu);
+
+	gtk_menu_item_set_submenu (GTK_MENU_ITEM (menu_item), menu);
+
+	return menu_item;
+}
+
+static gboolean
+create_menu_proxy (EggToolItem *item, EggAction *action)
+{
+	GtkWidget *menu_item;
+	char *menu_id;
+
+	LOG ("create_menu_proxy item %p, action %p", item, action)
+
+	menu_item = create_menu_item (action);
+
+	menu_id = g_strdup_printf ("ephy-topic-action-%d-menu-id",
+				   EPHY_TOPIC_ACTION (action)->priv->topic_id);
+
+	egg_tool_item_set_proxy_menu_item (item, menu_id, menu_item);
+
+	g_free (menu_id);
+
+	return TRUE;
 }
 
 static void
@@ -408,15 +483,28 @@ connect_proxy (EggAction *action, GtkWidget *proxy)
 {
 	GtkWidget *button;
 
+	LOG ("connect_proxy action %p, proxy %p", action, proxy)
+
 	(* EGG_ACTION_CLASS (parent_class)->connect_proxy) (action, proxy);
 
 	ephy_topic_action_sync_label (action, NULL, proxy);
 	g_signal_connect_object (action, "notify::label",
 			         G_CALLBACK (ephy_topic_action_sync_label), proxy, 0);
 
-	button = GTK_WIDGET (g_object_get_data (G_OBJECT (proxy), "button"));
-	g_signal_connect (button, "toggled",
-			  G_CALLBACK (button_toggled_cb), action);
+	if (EGG_IS_TOOL_ITEM (proxy))
+	{
+		g_signal_connect_object (proxy, "create_menu_proxy",
+					 G_CALLBACK (create_menu_proxy),
+					 action, 0);
+
+		button = GTK_WIDGET (g_object_get_data (G_OBJECT (proxy), "button"));
+		g_signal_connect (button, "toggled",
+				  G_CALLBACK (button_toggled_cb), action);
+
+		/* We want the menu to popup up on mouse down */
+		g_signal_connect (button, "pressed",
+				  G_CALLBACK (button_pressed_cb), action);
+	}
 }
 
 static void
@@ -484,6 +572,7 @@ ephy_topic_action_class_init (EphyTopicActionClass *class)
 
 	action_class->toolbar_item_type = EGG_TYPE_TOOL_ITEM;
 	action_class->create_tool_item = create_tool_item;
+	action_class->create_menu_item = create_menu_item;
 	action_class->connect_proxy = connect_proxy;
 
 	object_class->finalize = ephy_topic_action_finalize;
@@ -546,9 +635,9 @@ ephy_topic_action_init (EphyTopicAction *action)
 
 	bookmarks = ephy_shell_get_bookmarks (ephy_shell);
 	node = ephy_bookmarks_get_keywords (bookmarks);
-	g_signal_connect_object (node, "child_changed",
-				 G_CALLBACK (topic_child_changed_cb),
-				 action, 0);
+	ephy_node_signal_connect_object (node, EPHY_NODE_CHILD_CHANGED,
+				         (EphyNodeCallback) topic_child_changed_cb,
+				         G_OBJECT (action));
 }
 
 EggAction *
@@ -560,7 +649,7 @@ ephy_topic_action_new (const char *name, guint id)
 
 	bookmarks = ephy_shell_get_bookmarks (ephy_shell);
 
-	bmk = ephy_node_get_from_id (id);
+	bmk = ephy_bookmarks_get_from_id (bookmarks, id);
 	g_return_val_if_fail (bmk != NULL, NULL);
 
 	action = EGG_ACTION (g_object_new (EPHY_TYPE_TOPIC_ACTION,
