@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: window-commands.c,v 1.76.2.2 2003/09/23 22:23:22 chpe Exp $
+ *  $Id: window-commands.c,v 1.76.2.5 2004/01/19 11:59:42 chpe Exp $
  */
 
 #include <config.h>
@@ -750,9 +750,9 @@ window_cmd_help_about (EggAction *action,
 
 	static gchar *authors[] = {
 		"Marco Pesenti Gritti <marco@gnome.org>",
-		"Xan Lopez <xan@masilla.org>",
+		"Xan Lopez <xan@gnome.org>",
 		"David Bordoley <bordoley@msu.edu>",
-		"Christian Persch <chpe+epiphany@stud.uni-saarland.de>",
+		"Christian Persch <chpe@gnome.org>",
 		NULL
 	};
 
@@ -797,7 +797,10 @@ window_cmd_help_about (EggAction *action,
 		       strcmp (translator_credits, "translator_credits") != 0 ? translator_credits : NULL,
 		       logo);
 
-	g_object_unref (logo);
+	if (logo != NULL)
+	{
+		g_object_unref (logo);
+	}
 
 	gtk_window_set_transient_for (GTK_WINDOW (about),
 				      GTK_WINDOW (window));
